@@ -2,9 +2,10 @@ from __future__ import division, print_function
 
 from contextlib import contextmanager
 
-from .helper import unittest, PillowTestCase, hopper
 from PIL import Image, ImageDraw
 
+from .helper import PillowTestCase, hopper, unittest
+
 
 class TestImagingResampleVulnerability(PillowTestCase):
     # see https://github.com/python-pillow/Pillow/issues/1710
